@@ -120,6 +120,8 @@ SYSPROF_AVAILABLE_IN_ALL
 const SysprofCaptureFileChunk      *sysprof_capture_reader_read_file           (SysprofCaptureReader      *self);
 SYSPROF_AVAILABLE_IN_3_36
 const SysprofCaptureAllocation     *sysprof_capture_reader_read_allocation     (SysprofCaptureReader      *self);
+SYSPROF_AVAILABLE_IN_3_40
+const SysprofCaptureOverlay        *sysprof_capture_reader_read_overlay       (SysprofCaptureReader      *self);
 SYSPROF_AVAILABLE_IN_ALL
 bool                                sysprof_capture_reader_reset               (SysprofCaptureReader      *self);
 SYSPROF_AVAILABLE_IN_ALL
