@@ -26,7 +26,18 @@
 
 G_BEGIN_DECLS
 
-typedef struct _SysprofMapLookaside SysprofMapLookaside;
+typedef struct _SysprofMapOverlay
+{
+  const char *src;
+  const char *dst;
+} SysprofMapOverlay;
+
+typedef struct _SysprofMapLookaside
+{
+  GSequence    *seq;
+  GStringChunk *chunk;
+  GArray       *overlays;
+} SysprofMapLookaside;
 
 typedef struct
 {
@@ -37,12 +48,15 @@ typedef struct
   const gchar           *filename;
 } SysprofMap;
 
-SysprofMapLookaside *sysprof_map_lookaside_new    (void);
-void                 sysprof_map_lookaside_insert (SysprofMapLookaside   *self,
-                                                   const SysprofMap      *map);
-const SysprofMap    *sysprof_map_lookaside_lookup (SysprofMapLookaside   *self,
-                                                   SysprofCaptureAddress  address);
-void                 sysprof_map_lookaside_free   (SysprofMapLookaside   *self);
+SysprofMapLookaside *sysprof_map_lookaside_new      (void);
+void                 sysprof_map_lookaside_insert   (SysprofMapLookaside    *self,
+                                                     const SysprofMap       *map);
+void                 sysprof_map_lookaside_overlay  (SysprofMapLookaside    *self,
+                                                     const gchar            *src,
+                                                     const gchar            *dst);
+const SysprofMap    *sysprof_map_lookaside_lookup   (SysprofMapLookaside    *self,
+                                                     SysprofCaptureAddress   address);
+void                 sysprof_map_lookaside_free     (SysprofMapLookaside    *self);
 
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (SysprofMapLookaside, sysprof_map_lookaside_free)
 
