@@ -18,6 +18,7 @@
 
 #pragma once
 
+#include <glib.h>
 #include <sysprof-capture.h>
 
 G_BEGIN_DECLS
@@ -26,8 +27,10 @@ G_BEGIN_DECLS
 
 # include "sysprof-battery-source.h"
 # include "sysprof-callgraph-profile.h"
+# include "sysprof-capture-autocleanups.h"
 # include "sysprof-capture-gobject.h"
 # include "sysprof-capture-symbol-resolver.h"
+# include "sysprof-control-source.h"
 # include "sysprof-diskstat-source.h"
 # include "sysprof-elf-symbol-resolver.h"
 # include "sysprof-gjs-source.h"
@@ -39,6 +42,7 @@ G_BEGIN_DECLS
 # include "sysprof-memprof-profile.h"
 # include "sysprof-memprof-source.h"
 # include "sysprof-netdev-source.h"
+# include "sysprof-preload-source.h"
 # include "sysprof-process-model-item.h"
 # include "sysprof-process-model.h"
 # include "sysprof-profile.h"
